#!/usr/bin/env python
"""
Illustrate simple contour plotting, contours on an image with
a colorbar for the contours, and labelled contours.

See also contour_image.py.
"""
import matplotlib
import numpy as np
import matplotlib.cm as cm
import matplotlib.mlab as mlab
import matplotlib.pyplot as plt

matplotlib.rcParams['xtick.direction'] = 'out'
matplotlib.rcParams['ytick.direction'] = 'out'

ndim = 3
delta = 0.25
x1 = np.arange(-3.0, 3.0, delta)
x2 = np.arange(-2.0, 2.0, delta)
x3 = np.arange(-4.0, 4.0, delta)
#iv1, iv2, iv3 = np.mgrid[:x1.size,:x2.size,:x3.size]
#x1v = x1[iv1.flatten()]
#x2v = x2[iv2.flatten()]
#x3v = x3[iv3.flatten()]
#xtv=np.vstack((x1v,x2v,x3v)).T
def lnprob(x, mu, icov):
      diff = x-mu
      return -np.dot(diff,np.dot(icov,diff))/2.0
means = np.random.rand(ndim)
cov = 0.5 - np.random.rand(ndim ** 2).reshape((ndim, ndim))
cov = np.triu(cov)
cov += cov.T - np.diag(cov.diagonal())
cov = np.dot(cov,cov)
icov = np.linalg.inv(cov)
sz_n1=x1.shape[0]
sz_n2=x2.shape[0]
sz_n3=x3.shape[0]
z1=np.zeros([sz_n3,sz_n2,sz_n1])
for i1 in xrange(sz_n1-1):
   for i2 in xrange(sz_n2-1):
      for i3 in xrange(sz_n3-1):
         z1[i3,i2,i1] = lnprob(np.array([x1[i1],x2[i2],x3[i3]]), means, icov)
x1v,x3v=np.meshgrid(x1,x3)
# Create a simple contour plot with labels using default colors.  The
# inline argument to clabel will control whether the labels are draw
# over the line segments of the contour, removing the lines beneath
# the label
plt.figure()
CS = plt.contour(x1v, x3v, np.sum(z1,axis=1))
plt.clabel(CS, inline=1, fontsize=10)
plt.title('Simplest default with labels')
plt.savefig('contour{0:02d}.eps'.format(1))
