module mod_jam_rms
  implicit none
  integer::ngauss,ndata,nbins,npot,nlum,nrad,nang
  Real,allocatable,dimension(:)::x1,y1,dens_lum,sigma_lum,&
qintr_lum,dens_pot,sigma_pot,qintr_pot,beta,surf_lum,qobs_lum,&
surf_pot,qobs_pot,xbin,ybin,modrms
  Real,allocatable,dimension(:)::kani,s2_lum,q2_lum,&
e2_lum,s2q2_lum,s2_pot,e2_pot,beta_z
  Real::my_pi,Gval,inc,xnow,ynow,radeg,inc_rad,mbh,distance,pc,sigmapsf,&
pixsize,step,ci2,si2
  Parameter(my_pi=3.14159265259,Gval=0.00430237,radeg=57.295779531)
End module mod_jam_rms
!----------------------------------------
module mod_jammin
  implicit none
  integer::ndata,nfib,nlum,nhold,nfree
  Integer,allocatable,dimension(:)::indfib,ihold,ifree
  Real,allocatable,dimension(:)::surf_lum,sigma_lum,qobs_lum,xfib,yfib,gamfib,&
vdata,evdata,xhold
  Real::distance_now,pc_now,sigmapsf,pixsize,my_pi,int_MLmin,int_MLmax,ML_best,c_best
  Parameter(my_pi=3.14159265259)

End module mod_jammin
!----------------------------------------
Function janis2_jeans_mge_integrand(uval)
use mod_jam_rms
!This routine computes the integrand of Eq.(28) of Cappellari (2008)
!for a model with constant anisotropy sigma_R^2 = b*sigma_z^2 and <V_R*V_z> = 0.
!Not vectorized like the IDL version. Will be called by dqxgs.
Implicit None
Integer::i,j,k,n_kin,iseed1
Real::janis2_jeans_mge_integrand,sum1,x2,y2,u2,&
aterm,bterm,cterm,dterm,eterm,uval

x2=xnow*xnow
y2=ynow*ynow
u2=uval*uval
sum1=0.0
Do j=1,npot
   Do k=1,nlum
      aterm = 0.5*(u2/s2_pot(j) + 1.0/s2_lum(k)) ! equation (29)
      bterm = 0.5*(e2_pot(j)*u2*u2/(s2_pot(j)*(1.0 - e2_pot(j)*u2)) +&
e2_lum(k)/s2q2_lum(k)) ! equation (30)
      cterm = e2_pot(j) - s2q2_lum(k)/s2_pot(j) ! equation (22)
      dterm = 1.0 - kani(k)*q2_lum(k) - ((1.0 - kani(k))*cterm + &
e2_pot(j)*kani(k))*u2 ! equation (23)
      eterm = aterm + bterm*ci2
      sum1 = sum1+ dens_lum(k)*qintr_pot(j)*dens_pot(j)*u2 &
         * (s2q2_lum(k)*(ci2 + kani(k)*si2) + x2*si2*dterm) &
         * exp(-aterm*(x2 + y2*(aterm + bterm)/eterm)) / ((1.0 &
         - cterm*u2)*sqrt((1.0 - e2_pot(j)*u2)*eterm))
   End Do
End Do

janis2_jeans_mge_integrand=4.0*Gval*sum1*my_pi**1.5
return
end function janis2_jeans_mge_integrand
!----------------------------------------
function janis2_weighted_second_moment_squared(xin,yin)
use mod_jam_rms
Implicit None
Integer::ier,limit,leniw,lenw,last
Real::xin,yin,janis2_weighted_second_moment_squared,sb_mu2,&
epsabs,epsrel,abserr
Parameter(limit=20,lenw=920,leniw=60)
Integer::iwork(leniw)
Real::work(lenw)
External::janis2_jeans_mge_integrand
epsabs = 0.0d-3
epsrel = 1.0d-4
ier=0
xnow=xin
ynow=yin
!call qgaus(janis2_jeans_mge_integrand,0.0,1.0,sb_mu2)
call dqxgs(janis2_jeans_mge_integrand,0.0,1.0,epsabs,epsrel,sb_mu2,&
abserr,ier,limit,leniw,lenw,last,iwork,work)
if (ier /= 0 ) then
   write(*,*) "dqxgs error in subroutine janis2:",ier
   stop
endif
janis2_weighted_second_moment_squared=sb_mu2

return
end function janis2_weighted_second_moment_squared
!----------------------------------------
Subroutine janis2_second_moment(in_nbins,in_xin,in_yin,rmsModel)
! This routine gives the second V moment after convolution with a PSF.
! The convolution is done using interpolation of the model on a
! polar grid, as described in Appendix A of Cappellari (2008).
! vrms are fit on ellpitcal grid. Need a 2d interpolation 
! code to prep fft; we'll use polin2 but with 
! the 'regular' independent variables being the log radius and the 
! eccentric anomaly. Then need a 2d fft code. The best I can see is to 
! use rlft3 to fourier transform the data and kernal, multiply, and 
! make the reverse transform again with rlft3. Then 
! interpolate to fiber grid. This can use the regular, high 
! accuracy grid method of polin2(x1a,x2a,ya,m,n,x1,x2,y,dy).
use mod_jam_rms
Implicit None
Integer::i,j,k,k1,in_nbins,n1,n2,n1fft,n2fft,m,nint1,nint2,intp_ord,k2,k3,hn1fft
Parameter(intp_ord=3)
Real::in_xin(in_nbins),in_yin(in_nbins),rmsModel(in_nbins),&
rell(in_nbins),janis2_weighted_second_moment_squared,xcen,ycen
Real::qmed,mx,wm2,mge,rjunk1,rjunk2,rjunk3,rjunk4,rmax1,rmax2,r1,e1,&
rwei1,rwei2,step_ang,step_lograd,t1,t2,t3,t4,t5,sub_intp(intp_ord,intp_ord)
Real,allocatable,dimension(:)::samplogr,sampang,wm2pol,mgepol,&
xpol,ypol,tmpmu,xcar,ycar
Real,allocatable,dimension(:,:)::mupol,datafft_in,datafft_out,resp,&
wm2car,mgecar,wm2pol2d,mgepol2d,deriv_intpol1,deriv_intpol2,deriv_intcar,&
mucar
nbins=in_nbins
Allocate(x1(nbins),y1(nbins),samplogr(nrad),sampang(nang),xpol(nrad*nang),&
wm2pol(nrad*nang),mgepol(nrad*nang),ypol(nrad*nang),tmpmu(nrad*nang),mupol(nrad,nang),&
wm2pol2d(nrad,nang),mgepol2d(nrad,nang),deriv_intpol1(intp_ord,intp_ord),deriv_intpol2(intp_ord,intp_ord))
x1(:)=in_xin(:)
y1(:)=in_yin(:)

! Define parameters of polar grid for interpolation
!
qmed=sum(qobs_lum(:))/real(nlum)
rell(:) = sqrt(x1(:)**2 + (y1(:)/qmed)**2) ! Elliptical radius of input (x,y)

! Kernel step is 1/4 of largest value between sigma(min) and 1/2 pixel side.
! Kernel half size is the sum of 3*sigma(max) and 1/2 pixel diagonal.
!
if (sigmaPsf .gt. 1.0d-8 .and. pixsize .gt. 1.0d-8) then ! PSF convolution
    if (step .le. 0.0) then 
       step = max(pixSize/8.0,sigmapsf/4.0)
    end if
    mx = 3.0*sigmapsf + pixsize/sqrt(2.0)
else ! No convolution
   if (NRAD*NANG.ge.nbins) then
        rmsModel(:)=0.0
        Do j=1,nbins
            wm2 = janis2_weighted_second_moment_squared(x1(j),y1(j))
            mge = sum(surf_lum(:)*exp(-0.5/sigma_lum(:)**2 * (x1(j)**2 &
+ (y1(j)/qobs_lum(:))**2)))
            rmsModel(j) = sqrt(wm2/mge)
        End Do
        return ! Skip the interpolation when computing just a few points        
   endif
   step = minval(rell)+1.d1 !minimum of 10 pc
   mx = 0.0
endif
! Make linear grid in log of elliptical radius RAD and eccentric anomaly ANG
! See Appendix A
!
rmax1 = 1.3*maxval(rell) + mx ! Major axis of ellipse containing all data + convolution
rmax2 = maxval(rell) + mx ! Major axis of ellipse containing all data + convolution
rjunk1=log(step)
rjunk2=log(rmax1)
Do i=1,nrad
   samplogr(i)=rjunk1+(rjunk2-rjunk1)*real(i-1)/real(nrad-1)
End Do
rjunk1=0.0
rjunk2=my_pi/2.0
Do i=1,nang
   sampang(i)=rjunk1+(rjunk2-rjunk1)*real(i-1)/real(nang-1)
End Do
Do i=1,nrad
   Do j=1,nang
      xpol(j+(i-1)*nang)=exp(samplogr(i))*cos(sampang(j))
      ypol(j+(i-1)*nang)=exp(samplogr(i))*sin(sampang(j))*qmed
   End Do
End Do

! The model Vrms computation is only performed on the polar grid
! which is then used to interpolate the values at any other location
!
wm2pol(:)=0.0
mgepol(:)=0.0
Do j=1,nrad*nang
   wm2Pol(j) = janis2_weighted_second_moment_squared(xpol(j), ypol(j))
   mgePol(j) = sum(surf_lum(:)*exp(-0.5/sigma_lum(:)**2*(&
xpol(j)**2+(ypol(j)/qobs_lum(:))**2)))
end do
step_ang=sampang(2)-sampang(1)
step_lograd=samplogr(2)-samplogr(1)

if (sigmaPsf.gt.0.0.and.pixSize.gt.0.0) then ! PSF convolution
!
   Do i=1,nrad
      Do j=1,nang
         wm2pol2d(i,j)=wm2pol(j+(i-1)*nang)
         mgepol2d(i,j)=mgepol(j+(i-1)*nang)
      End Do
   End Do
!   call splie2(samplogr,sampang,wm2pol2d,nrad,nang,deriv_intpol1)
!   call splie2(samplogr,sampang,mgepol2d,nrad,nang,deriv_intpol2)
    n1 = ceiling(2.*rmax2/step)
    n2 = ceiling(2.*rmax2*qmed/step)
    m = ceiling(2*mx/step)
    if (mod(m,2).eq.0) m=m+1
    n1fft=2**ceiling(log(real(n1))/log(2.))
    n2fft=2**ceiling(log(real(n2))/log(2.))
    hn1fft=n1fft/2
    Allocate(wm2car(n1,n2),mgecar(n1,n2),datafft_in(n1,n2),datafft_out(n1,n2),resp(m,m),&
xcar(n1),ycar(n2))
    call mk_kernel(pixsize,sigmapsf,step,m,resp)
    xcen=real(n1+1)/2.
    ycen=real(n2+1)/2.
    Do i=1,n1
       xcar(i)=real(i-xcen)*step
    End Do
    Do i=1,n2
       ycar(i)=real(i-ycen)*step
    End Do
    Do i=1,n1
       xnow=xcar(i)
       Do j=1,n2
          ynow=ycar(j)
!          r1 = 0.5*log(xnow**2 + (ynow/qmed)**2) ! Log elliptical radius of input (x,y)
          r1=xnow**2 + (ynow/qmed)**2
          if (r1.lt.1.d-6) r1=r1+1.d-6
          r1 =0.5*log(r1)
          e1 = atan2(abs(ynow/qmed),abs(xnow))    ! Eccentric anomaly of input (x,y)
          if (r1.gt.samplogr(nrad)) r1=samplogr(nrad)
          if (r1.lt.samplogr(1)) r1=samplogr(1)
          nint1=floor((r1-samplogr(1))/step_lograd+1)
          nint2=floor((e1-sampang(1))/step_ang+1)
          k2=min(max(nint1-(intp_ord-1)/2,1),nrad+1-intp_ord)
          k3=min(max(nint2-(intp_ord-1)/2,1),nang+1-intp_ord)
          Do k=1,intp_ord
             sub_intp(k,1:intp_ord)=wm2pol2d(k2+k-1,k3:k3+intp_ord-1)
          End Do
!          rjunk1=(r1-samplogr(nint1))/step_lograd
!          rjunk2=(e1-sampang(nint2))/step_ang
!          rjunk3=(1.0-rjunk1)*(1.0-rjunk2)*wm2pol2d(nint1,nint2)+&
!rjunk1*(1.0-rjunk2)*wm2pol2d(nint1+1,nint2)+(1.0-rjunk1)*rjunk2*&
!wm2pol2d(nint1,nint2+1)+rjunk1*rjunk2*wm2pol2d(nint1+1,nint2+1)
          call splie2(samplogr(k2:k2+intp_ord-1),sampang(k3:k3+intp_ord-1),sub_intp,&
intp_ord,intp_ord,deriv_intpol1)
          call splin2(samplogr(k2:k2+intp_ord-1),sampang(k3:k3+intp_ord-1),sub_intp,deriv_intpol1,&
intp_ord,intp_ord,r1,e1,rjunk3)
!          call polin2(samplogr(k2:k2+intp_ord-1),sampang(k3:k3+intp_ord-1),sub_intp,intp_ord,intp_ord,r1,e1,rjunk3,rjunk4)
          wm2car(i,j) = rjunk3
          Do k=1,intp_ord
             sub_intp(k,1:intp_ord)=mgepol2d(k2+k-1,k3:k3+intp_ord-1)
          End Do
          call splie2(samplogr(k2:k2+intp_ord-1),sampang(k3:k3+intp_ord-1),sub_intp,&
intp_ord,intp_ord,deriv_intpol2)
          call splin2(samplogr(k2:k2+intp_ord-1),sampang(k3:k3+intp_ord-1),sub_intp,deriv_intpol2,&
intp_ord,intp_ord,r1,e1,rjunk3)
!          rjunk3=(1.0-rjunk1)*(1.0-rjunk2)*mgepol2d(nint1,nint2)+&
!rjunk1*(1.0-rjunk2)*mgepol2d(nint1+1,nint2)+(1.0-rjunk1)*rjunk2*&
!mgepol2d(nint1,nint2+1)+rjunk1*rjunk2*mgepol2d(nint1+1,nint2+1)
!          call splin2(samplogr,sampang,mgepol2d,deriv_intpol2,nrad,nang,r1,e1,rjunk3)
!          call jja_polin2(samplogr,sampang,mgepol2d,nrad,nang,r1,e1,rjunk3,rjunk4)
!          call polin2(samplogr(k2:k2+intp_ord-1),sampang(k3:k3+intp_ord-1),sub_intp,intp_ord,intp_ord,r1,e1,rjunk3,rjunk4)
          mgecar(i,j) = rjunk3
       End Do
    End DO
    ! Seeing and aperture convolution with equation (A3)
    !
    call numrec_2d_convol(m,resp,n1,n2,n1fft,n2fft,hn1fft,wm2car,datafft_in)
    call numrec_2d_convol(m,resp,n1,n2,n1fft,n2fft,hn1fft,mgecar,datafft_out)
    Do i=1,n1
       Do j=1,n2
          datafft_in(i,j)=datafft_in(i,j)/datafft_out(i,j)
       End Do
    End Do
    Deallocate(wm2car,mgecar,datafft_out,resp)
    Allocate(deriv_intcar(n1,n2),mucar(n1,n2))
!Fix possible rounding problems
    rjunk1=1.0d20
    Do i=1,n1
       Do j=1,n2
          if (datafft_in(i,j).gt.0.0.and.datafft_in(i,j).le.rjunk1) rjunk1=datafft_in(i,j)
       End Do
    End Do
    Do i=1,n1
       Do j=1,n2
          if (datafft_in(i,j).le.0.0) datafft_in(i,j)=rjunk1
          mucar(i,j)=sqrt(datafft_in(i,j))
       End Do
    End Do
!    call splie2(xcar,ycar,mucar,n1,n2,deriv_intcar)
    Do i=1,nbins
!       call splin2(xcar,ycar,mucar,deriv_intcar,n1,n2,x1(i),y1(i),rjunk3)
!       call jja_polin2(xcar,ycar,mucar,n1,n2,x1(i),y1(i),rjunk3,rjunk4)
!       rmsModel(i) = rjunk3
       nint1=floor((x1(i)-xcar(1))/step+1)
       nint2=floor((y1(i)-ycar(1))/step+1)
       k2=min(max(nint1-(intp_ord-1)/2,1),n1+1-intp_ord)
       k3=min(max(nint2-(intp_ord-1)/2,1),n2+1-intp_ord)
       Do k=1,intp_ord
          sub_intp(k,1:intp_ord)=mucar(k2+k-1,k3:k3+intp_ord-1)
       End Do
       call splie2(xcar(k2:k2+intp_ord-1),ycar(k3:k3+intp_ord-1),sub_intp,&
intp_ord,intp_ord,deriv_intpol1)
       call splin2(xcar(k2:k2+intp_ord-1),ycar(k3:k3+intp_ord-1),sub_intp,deriv_intpol1,&
intp_ord,intp_ord,x1(i),y1(i),rjunk3)
!       call polin2(xcar(k2:k2+intp_ord-1),ycar(k3:k3+intp_ord-1),sub_intp,intp_ord,intp_ord,x1(i),y1(i),rjunk3,rjunk4)
      rmsModel(i)=rjunk3
!       rmsModel(i) = mucar(min(n1,max(1,nint((x1(i)-xcar(1))/step)))&
!,min(n2,max(1,nint((y1(i)-ycar(1))/step))))
    End Do
!stop
    ! Interpolate convolved image at observed apertures.
    ! Aperture integration was already included in the kernel.
    !
    deallocate(datafft_in,xcar,ycar,deriv_intcar,mucar)

else ! No PSF convolution: just interpolate values
!
   tmpmu(:) = wm2Pol(:)/mgePol(:)
   rjunk1=1.0d20
   Do i=1,nrad*nang
      if (tmpmu(i).gt.0.0.and.tmpmu(i).le.rjunk1) rjunk1=tmpmu(i)
   End Do
   Do i=1,nrad*nang
      if (tmpmu(i).le.0.0) tmpmu(i)=rjunk1
   End Do
   Do i=1,nrad
      Do j=1,nang
         mupol(i,j)=sqrt(tmpmu(j+(i-1)*nang))
      End Do
   End Do
!   call splie2(samplogr,sampang,mupol,nrad,nang,deriv_intpol1)
   Do i=1,nbins
       rjunk1=x1(i)**2 + (y1(i)/qmed)**2
       if (rjunk1.lt.1.d-6) rjunk1=rjunk1+1.d-6
       rjunk1 =0.5*log(rjunk1)
      rjunk2 = atan2(abs(y1(i)/qmed),abs(x1(i)))    ! Eccentric anomaly of input (x,y)
          if (rjunk1.gt.samplogr(nrad)) rjunk1=samplogr(nrad)
          if (rjunk1.lt.samplogr(1)) rjunk1=samplogr(1)
          nint1=floor((rjunk1-samplogr(1))/step_lograd+1)
          nint2=floor((rjunk2-sampang(1))/step_ang+1)
          k2=min(max(nint1-(intp_ord-1)/2,1),nrad+1-intp_ord)
          k3=min(max(nint2-(intp_ord-1)/2,1),nang+1-intp_ord)
          Do k=1,intp_ord
             sub_intp(k,1:intp_ord)=mupol(k2+k-1,k3:k3+intp_ord-1)
          End Do
          call splie2(samplogr(k2:k2+intp_ord-1),sampang(k3:k3+intp_ord-1),sub_intp,&
intp_ord,intp_ord,deriv_intpol1)
          call splin2(samplogr(k2:k2+intp_ord-1),sampang(k3:k3+intp_ord-1),sub_intp,deriv_intpol1,&
intp_ord,intp_ord,rjunk1,rjunk2,rjunk3)
!          call polin2(samplogr(k2:k2+intp_ord-1),sampang(k3:k3+intp_ord-1),sub_intp,intp_ord,intp_ord,rjunk1,rjunk2,rjunk3,rjunk4)
!      call splin2(samplogr,sampang,mupol,deriv_intpol1,nrad,nang,rjunk1,rjunk2,rjunk3)
!      call jja_polin2(samplogr,sampang,mupol,nrad,nang,rjunk1,rjunk2,rjunk3,rjunk4)
      rmsModel(i) = rjunk3
   End Do
endif
Deallocate(x1,y1,samplogr,sampang,wm2pol,mgepol,xpol,ypol,tmpmu,mupol,&
wm2pol2d,mgepol2d,deriv_intpol1,deriv_intpol2)
return
end Subroutine janis2_second_moment
!----------------------------------------
Subroutine mk_kernel(pixsize,sigmapsf,pix_sc,m,resp)
Implicit None
Integer::i,j,k,m
Real::erf,resp(m,m),pix_sc,sigmapsf,dx,pixsize,rjunk1,rjunk2 
!pix_sc, etc in arcsec
dx=pixsize/2.0
rjunk1=sqrt(2.0)*sigmapsf
rjunk2=0.0
Do i=1,m
   Do j=1,m
      resp(i,j)=(erf((dx-real(i-1)*pix_sc)/rjunk1)+erf((dx+&
real(i-1)*pix_sc)/rjunk1))*(erf((dx-real(j-1)*pix_sc)/rjunk1)+erf((dx+&
real(j-1)*pix_sc)/rjunk1))
      rjunk2=rjunk2+resp(i,j)
   End Do
End Do
Do i=1,m
   Do j=1,m
      resp(i,j)=resp(i,j)/rjunk2
   End Do
End Do

return
End Subroutine mk_kernel
!----------------------------------------
Subroutine numrec_2d_convol(m,resp,n1,n2,n1fft,n2fft,hn1fft,data_in,data_out)
!Routine to handle 2d convolution for real data. rlft3 does
!most of the work. Note bothering to ensure a zero-pad of
!the data as we'll have enough buffer in the original data to not care
!about the edges (anyway, convolution doesn't sample data
!outside the input windown anyhow, so the edges will be
!wrong either way). This routine will round up the images sizes
!to be a necessary 2 multiple and will zero pad the reponse as need
!be. m must be odd. resp is just the positive quadrant of the
!full reponse, and we assume it to be symmetric. Must precompute
!factor of 2 image sizes on call.
Implicit None
Integer::m,n1,n2,n1fft,n2fft,n3fft,i,j,k,hn1fft
Parameter(n3fft=1)
Real::resp(m,m),data_in(n1,n2),data_out(n1,n2),fac1,rjunk1
Real,Dimension(n1fft,n2fft,n3fft)::data1_fft,data2_fft
Complex,Dimension(hn1fft,n2fft,n3fft)::spec1,spec2
Complex,Dimension(n2fft,n3fft)::speq1,speq2
!Complex,Dimension(n1fft*n2fft*n3fft/2)::zpec1,zpec2
!Complex,Dimension(n1fft*n2fft)::zpeq1,zpeq2
!Equivalence (data1_fft,spec1), (data2_fft,spec2)
!Equivalence (data1_fft,spec1,zpec1), (data2_fft,spec2,zpec2),
!(speq1,zpeq1), (speq2,zpeq2)
fac1=2.0/real(n1fft*n2fft*n3fft) !factor to get normalized inverse
data1_fft(:,:,:)=0.0
data2_fft(:,:,:)=0.0

k=1
Do i=1,n1
   Do j=1,n2
      data1_fft(i,j,k)=data_in(i,j)
   End Do
End Do
rjunk1=0.0
Do i=1,m
   Do j=1,m
      data2_fft(i,j,k)=resp(i,j)
   End Do
   Do j=2,m
      data2_fft(i,n2fft+2-j,k)=resp(i,j)
   End Do
End Do
Do i=2,m
   Do j=1,m
      data2_fft(n1fft+2-i,j,k)=resp(i,j)
   End Do
   Do j=2,m
      data2_fft(n1fft+2-i,n2fft+2-j,k)=resp(i,j)
   End Do
End Do
rjunk1=sum(data2_fft)
data2_fft=data2_fft/rjunk1

Do i=1,hn1fft
   spec1(i,:,:)=CMPLX(data1_fft(2*i-1,:,:),data1_fft(2*i,:,:))
   spec2(i,:,:)=CMPLX(data2_fft(2*i-1,:,:),data2_fft(2*i,:,:))
End Do

call rlft3(spec1,speq1,n1fft,n2fft,n3fft,1)
call rlft3(spec2,speq2,n1fft,n2fft,n3fft,1)
Do j=1,n2fft
   speq1(j,k)=fac1*speq1(j,k)*speq2(j,k)
   Do i=1,hn1fft
      spec1(i,j,k)=fac1*spec1(i,j,k)*spec2(i,j,k)
   End Do
End Do

call rlft3(spec1,speq1,n1fft,n2fft,n3fft,-1)
Do i=1,hn1fft
   data1_fft(2*i-1,:,:)=REALPART(spec1(i,:,:))
   data1_fft(2*i,:,:)=IMAGPART(spec1(i,:,:))
End Do
Do i=1,n1
   Do j=1,n2
      data_out(i,j)=data1_fft(i,j,k)
   End Do
End Do
return
end subroutine numrec_2d_convol
!----------------------------------------
Subroutine jam_axisymmetric_rms(in_nlum,in_surf_lum,in_sigma_lum,in_qobs_lum,&
in_npot,in_surf_pot,in_sigma_pot,in_qobs_pot,in_inc,in_mbh,in_distance,in_ndata,&
in_xbin,in_ybin,in_modrms,in_beta_z,in_sigmapsf,in_pixsize)
use mod_jam_rms
Implicit None
Integer::i,j,k,in_nlum,in_ndata,in_npot
Real::rbh,normpsf,in_inc,in_mbh,in_distance,in_sigmapsf,in_pixsize
Real::in_surf_lum(in_nlum),in_sigma_lum(in_nlum),in_qobs_lum(in_nlum),&
in_beta_z(in_nlum),in_surf_pot(in_npot),in_sigma_pot(in_npot),in_qobs_pot(in_npot),&
in_xbin(in_ndata),in_ybin(in_ndata),in_modrms(in_ndata)
if (in_mbh.gt.0.0) then
   npot=in_npot+1
else
   npot=in_npot
endif
nlum=in_nlum
ndata=in_ndata
Allocate(surf_lum(nlum),sigma_lum(nlum),qobs_lum(nlum),dens_lum(nlum),&
qintr_lum(nlum),beta_z(nlum),surf_pot(npot),sigma_pot(npot),qobs_pot(npot),&
dens_pot(npot),qintr_pot(npot),xbin(ndata),ybin(ndata))
Allocate(kani(nlum),s2_lum(nlum),q2_lum(nlum),e2_lum(nlum),&
s2q2_lum(nlum),s2_pot(npot),e2_pot(npot))

step=0.0
nrad=50
nang=10
rbh=0.01 !arcsec
normpsf=1.0
distance=in_distance
inc=in_inc
pc = distance*my_pi/0.648 ! Constant factor to convert arcsec --> pc

surf_lum(:)=in_surf_lum(:)
sigma_lum(:)=in_sigma_lum(:)*pc
qobs_lum(:)=in_qobs_lum(:)

beta_z(:)=in_beta_z(:)
xbin(:)=in_xbin(:)*pc
ybin(:)=in_ybin(:)*pc
inc_rad=inc/radeg
ci2=cos(inc_rad)**2
si2=sin(inc_rad)**2
sigmapsf=in_sigmapsf*pc
pixsize=in_pixsize*pc
if (in_mbh.gt.0.0) then
   surf_pot(1:npot-1)=in_surf_pot(:)
   sigma_pot(1:npot-1)=in_sigma_pot(:)*pc
   qobs_pot(1:npot-1)=in_qobs_pot(:)
   surf_pot(npot)=in_mbh/(2.0*my_pi*(rbh*pc)**2)
   sigma_pot(npot)=rbh*pc
   qobs_pot(npot)=1.0
else
   surf_pot(:)=in_surf_pot(:)
   sigma_pot(:)=in_sigma_pot(:)*pc
   qobs_pot(:)=in_qobs_pot(:)
endif

Do i=1,nlum
   if (qobs_lum(i)**2.le.cos(inc_rad)**2) then
      write(*,*) "Error in inclination and axial ratios", qobs_lum(i), inc
      stop
   else
      qintr_lum(i)=sqrt(qobs_lum(i)**2-cos(inc_rad)**2)/sin(inc_rad)
      dens_lum(i)=surf_lum(i)*qobs_lum(i)/(sigma_lum(i)*&
qintr_lum(i)*sqrt(2.0*my_pi))
      if (qintr_lum(i).le.0.0) then
         write(*,*) "Error in inclination and axial ratios", qintr_lum(i), inc
         stop
      endif
   endif
End Do
Do i=1,npot
   if (qobs_pot(i)**2.le.cos(inc_rad)**2) then
      write(*,*) "Error in inclination and axial ratios", qobs_pot(i), inc
      stop
   else
      qintr_pot(i)=sqrt(qobs_pot(i)**2-cos(inc_rad)**2)/sin(inc_rad)
      dens_pot(i)=surf_pot(i)*qobs_pot(i)/(sigma_pot(i)*&
qintr_pot(i)*sqrt(2.0*my_pi))
      if (qintr_pot(i).le.0.0) then
         write(*,*) "Error in inclination and axial ratios", qintr_pot(i), inc
         stop
      endif
   endif
End Do
Do i=1,nlum
   kani(i)=1.0/(1.0-beta_z(i))
   s2_lum(i)=sigma_lum(i)**2
   q2_lum(i)=qintr_lum(i)**2
   e2_lum(i)=1.0-q2_lum(i)
   s2q2_lum(i)=s2_lum(i)*q2_lum(i)
End Do
Do i=1,npot
   s2_pot(i)=sigma_pot(i)**2
   e2_pot(i)=1.0-qintr_pot(i)**2
End Do

call janis2_second_moment(ndata,xbin,ybin,in_modrms)
Deallocate(surf_lum,sigma_lum,qobs_lum,qintr_lum,dens_lum,beta_z,&
surf_pot,sigma_pot,qobs_pot,qintr_pot,dens_pot,xbin,ybin)
Deallocate(kani,s2_lum,q2_lum,e2_lum,s2q2_lum,s2_pot,e2_pot)
return
end subroutine jam_axisymmetric_rms
!----------------------------------------
Subroutine gNFW_params(cval,M200,dm_gamma,rho_cen,rad_c)
Implicit None
!Outputs the density at r_s and r_s (as rad_c) in Msol/pc^3 and pc
Real::cval,M200,tmp_gamma,zh_betai,dm_zeta,zh_gammln,rho_cen,rad_c,rho_crit,&
my_pi,dm_gamma
Parameter(my_pi=3.14159265259,rho_crit=1.4384d-7) !units Msol/pc^3

if (dm_gamma.lt.1.0) then
   dm_zeta=((1.0+cval)/cval)**(dm_gamma-2.0)*(2.0*dm_gamma*cval-3.0&
*cval+dm_gamma-2.0)/(dm_gamma*dm_gamma-3.0*dm_gamma+2.0)/cval+exp(zh_gammln(2.0&
-dm_gamma)-zh_gammln(1.0-dm_gamma))*zh_betai(1.0-dm_gamma,0.0,cval/(cval+1.0))/(1.0-dm_gamma)
else if (dm_gamma .eq. 1.0) then
   dm_zeta=log(1.0+cval)-cval/(1.0+cval)
else
   tmp_gamma=my_pi/sin(my_pi*(1.0-dm_gamma))/exp(zh_gammln(dm_gamma))
   dm_zeta=((1.0+cval)/cval)**(dm_gamma-2.0)*(2.0*dm_gamma*cval-3.0&
*cval+dm_gamma-2.0)/(dm_gamma*dm_gamma-3.0*dm_gamma+2.0)/cval+(exp(zh_gammln(2.0&
-dm_gamma))/tmp_gamma)*zh_betai(1.0-dm_gamma,0.0,cval/(cval+1.0))/(1.0-dm_gamma)
end if
rho_cen=rho_crit*(200.d0/3.d0)*(cval)**3/dm_zeta
rad_c=(3.0*M200/(800.0*my_pi*rho_crit*cval**3))**(1.0/3.0)

return
end subroutine gNFW_params
!----------------------------------------
Subroutine gNFW_mge(cval,M200,dm_gamma,rmin,rmax,ngauss,surf_dm,sigma_dm,qobs_dm)
Implicit None
!Get mge terms for a particular gNFW parameter set
Integer::nmax,ngauss,nrad,i,nkeep,ierr_sys
Parameter(nmax=15,nrad=101)
Real::cval,M200,rho_cen,rad_c,dm_gamma,surf_dm(nmax),sigma_dm(nmax),qobs_dm(nmax),&
logrmin,logrmax,rsamp(nrad),rhosamp(nrad),sol(2,nmax),rmin,rmax
!Parameter(logrmin=1.0,logrmax=6.0) !in pc
logrmin=log10(rmin)
logrmax=log10(rmax)
call gNFW_params(cval,M200,dm_gamma,rho_cen,rad_c)
Do i=1,nrad
   rsamp(i)=10.**(logrmin+(logrmax-logrmin)*real(i-1)/real(nrad-1))
   rhosamp(i)=rho_cen/((rsamp(i)/rad_c)**(dm_gamma)*(1.d0+rsamp(i)/rad_c)**(3.d0-dm_gamma))
End Do
call mge_fit_1d(nrad,rsamp,rhosamp,nmax,sol,ngauss)
surf_dm(:)=0.0
sigma_dm(:)=0.0
qobs_dm(:)=0.0
Do i=1,ngauss
   surf_dm(i)=sol(2,i)
   sigma_dm(i)=sol(1,i)
   qobs_dm(i)=1.0
End Do
return
end subroutine gNFW_mge
!----------------------------------------
Subroutine gen_jam_model(ML_now,inc_now,beta_now,cval,log_M200,dm_gamma,&
distance_now,sigmapsf,pixsize,ndata,&
vrmsmodel,vfib,nfib,xfib,yfib,gamfib,indfib,nlum,surf_lum,sigma_lum,qobs_lum,iHI,&
n_mge_HI,surf_mge_HI,sigma_mge_HI,qobs_mge_HI)
Implicit None
!Also returns original per fiber measurment/predictions.
!call a JAM model for a single set of parameters. Considers that many fibers
!make up a bin. Needs a map of fiber/bin ownership to weights to do this, but 
!this code uses those weights. Generates a single model. Similar version for 
!chisq feeding to lancelot.
Integer::i,j,k,ndata,nfib,nlum,ngauss,nmax,npot,iHI,n_mge_HI
Parameter(nmax=15)
Real::ML_now,inc_now,beta_now,cval,M200,dm_gamma,mbh_now,distance_now,sigmapsf,pixsize,log_M200
Real::vrmsmodel(ndata),xfib(nfib),yfib(nfib),vfib(nfib),gamfib(nfib),surf_lum(nlum),&
sigma_lum(nlum),qobs_lum(nlum),surf_tmp(nmax),sigma_tmp(nmax),qobs_tmp(nmax),tmp_sum1(ndata),&
tmp_sum2(ndata),pc_now,my_pi,rmin,rmax,beta_lum(nlum),surf_mge_HI(n_mge_HI),sigma_mge_HI(n_mge_HI),&
qobs_mge_HI(n_mge_HI),lum_tmp(nmax),qi_tmp(nmax)
Parameter(my_pi=3.14159265259)
Real,allocatable,dimension(:)::surf_pot,sigma_pot,qobs_pot
Parameter(mbh_now=0.0)
Integer::indfib(nfib)

M200=10.0**log_M200
pc_now=distance_now*my_pi/0.648
rmin=max(pc_now*pixsize/10.0,10.0) !min 10 pc size
rmax=pc_now*sqrt(maxval(xfib(:)**2+yfib(:)**2))*10.0
call gNFW_mge(cval,M200,dm_gamma,rmin,rmax,ngauss,surf_tmp,sigma_tmp,qobs_tmp)
Do i=1,ngauss
   sigma_tmp(i)=sigma_tmp(i)/pc_now
!   lum_tmp(i)=2.0*my_pi*surf_tmp(i)*sigma_tmp(i)**2
End Do
!call solve_qmge(ngauss,lum_tmp,sigma_tmp,qi_tmp,1.6,0.4,40.0)
!Do i=1,ngauss
!   qobs_tmp(i)=sqrt((qi_tmp(i)*sin(inc_now*my_pi/180.0))**2+cos(inc_now*my_pi/180.0)**2)
!   surf_tmp(i)=surf_tmp(i)/qobs_tmp(i)
!End Do

if (iHI.eq.1) then
npot=nlum+ngauss+n_mge_HI
Allocate(surf_pot(npot),sigma_pot(npot),qobs_pot(npot))
surf_pot(1:nlum)=ML_now*surf_lum(1:nlum)
sigma_pot(1:nlum)=sigma_lum(1:nlum)
qobs_pot(1:nlum)=qobs_lum(1:nlum)
surf_pot(nlum+1:nlum+ngauss)=surf_tmp(1:ngauss)
sigma_pot(nlum+1:nlum+ngauss)=sigma_tmp(1:ngauss) !Done above, /pc_now
qobs_pot(nlum+1:nlum+ngauss)=qobs_tmp(1:ngauss)
!Values for NGC 2976
Do i=1,n_mge_HI
   surf_pot(nlum+ngauss+i)=surf_mge_HI(i)
   sigma_pot(nlum+ngauss+i)=sigma_mge_HI(i)
   qobs_pot(nlum+ngauss+i)=qobs_mge_HI(i)
End Do
else
npot=nlum+ngauss
Allocate(surf_pot(npot),sigma_pot(npot),qobs_pot(npot))
surf_pot(1:nlum)=ML_now*surf_lum(1:nlum)
sigma_pot(1:nlum)=sigma_lum(1:nlum)
qobs_pot(1:nlum)=qobs_lum(1:nlum)
surf_pot(nlum+1:nlum+ngauss)=surf_tmp(1:ngauss)
sigma_pot(nlum+1:nlum+ngauss)=sigma_tmp(1:ngauss) !Done above, /pc_now
qobs_pot(nlum+1:nlum+ngauss)=qobs_tmp(1:ngauss)
endif
beta_lum(:)=beta_now

call jam_axisymmetric_rms(nlum,surf_lum,sigma_lum,qobs_lum,&
npot,surf_pot,sigma_pot,qobs_pot,inc_now,mbh_now,distance_now,nfib,&
xfib,yfib,vfib,beta_lum,sigmapsf,pixsize)

tmp_sum1(:)=0.0
tmp_sum2(:)=0.0
Do i=1,nfib
   tmp_sum1(indfib(i))=tmp_sum1(indfib(i))+vfib(i)**2*gamfib(i)
   tmp_sum2(indfib(i))=tmp_sum2(indfib(i))+gamfib(i)
End Do
Do i=1,ndata
   vrmsmodel(i)=sqrt(tmp_sum1(i)/tmp_sum2(i))
End Do
Deallocate(surf_pot,sigma_pot,qobs_pot)
return
end subroutine gen_jam_model
!----------------------------------------
Subroutine fmin_jam(x_in,fx,i_in)
use mod_jammin
Implicit None
!call a JAM model for a single set of parameters. Considers that many fibers
!make up a bin. Needs a map of fiber/bin ownership to weights to do this, but 
!this code uses those weights. Generates a single model. Similar version for 
!chisq feeding to lancelot.
Integer::i,j,k,ndum,nmax,i_in,npot,ngauss
Parameter(nmax=15)
real::x_in(:), rjunk1,xcomb(6)
real::fx
Real::ML_now,inc_now,beta_now,cval,M200,dm_gamma,mbh_now,surf_tmp(nmax),&
sigma_tmp(nmax),qobs_tmp(nmax),rmin,rmax
Real,allocatable,dimension(:)::surf_pot,sigma_pot,qobs_pot,tmp_sum1,tmp_sum2,&
vrmsmodel,vfib,beta_lum
Parameter(mbh_now=0.0)

!ML_now=x_in(1)
!inc_now=x_in(2)
!beta_now=x_in(3)
!cval=x_in(4)
!M200=10.**x_in(5)
!dm_gamma=x_in(6)
!Method to map arbitrary minization
Do i=1,nfree
   xcomb(ifree(i))=x_in(i)
End Do
if (nhold.gt.0) then
   Do i=1,nhold
      xcomb(ihold(i))=xhold(i)
   End Do
end if
ML_now=xcomb(1)
inc_now=xcomb(2)
beta_now=xcomb(3)
cval=xcomb(4)
M200=10.**xcomb(5)
dm_gamma=xcomb(6)

fx=0.0
!write(*,*) "Iteration ", x_in(:)
pc_now=distance_now*my_pi/0.648


rmin=max(pc_now*pixsize/10.0,10.0) !min 10 pc size
rmax=pc_now*sqrt(maxval(xfib(:)**2+yfib(:)**2))*10.0
call gNFW_mge(cval,M200,dm_gamma,rmin,rmax,ngauss,surf_tmp,sigma_tmp,qobs_tmp)
npot=nlum+ngauss
Allocate(surf_pot(npot),sigma_pot(npot),qobs_pot(npot),&
tmp_sum1(ndata),tmp_sum2(ndata),vrmsmodel(ndata),vfib(nfib),beta_lum(nlum))
surf_pot(1:nlum)=ML_now*surf_lum(1:nlum)
sigma_pot(1:nlum)=sigma_lum(1:nlum)
qobs_pot(1:nlum)=qobs_lum(1:nlum)
beta_lum(:)=beta_now
i=nlum+1
j=nlum+ngauss
surf_pot(i:j)=surf_tmp(1:ngauss)
sigma_pot(i:j)=sigma_tmp(1:ngauss)/pc_now
qobs_pot(i:j)=qobs_tmp(1:ngauss)
call jam_axisymmetric_rms(nlum,surf_lum,sigma_lum,qobs_lum,&
npot,surf_pot,sigma_pot,qobs_pot,inc_now,mbh_now,distance_now,nfib,&
xfib,yfib,vfib,beta_lum,sigmapsf,pixsize)
tmp_sum1(:)=0.0
tmp_sum2(:)=0.0
Do i=1,nfib
   tmp_sum1(indfib(i))=tmp_sum1(indfib(i))+vfib(i)**2*gamfib(i)
   tmp_sum2(indfib(i))=tmp_sum2(indfib(i))+gamfib(i)
End Do
Do i=1,ndata
   vrmsmodel(i)=sqrt(tmp_sum1(i)/tmp_sum2(i))
   fx=fx+((vrmsmodel(i)-vdata(i))/evdata(i))**2
End Do
fx=fx/real(ndata)
!write(*,*) x_in(:), fx
Deallocate(surf_pot,sigma_pot,qobs_pot,tmp_sum1,tmp_sum2,vrmsmodel,vfib,beta_lum)
return
end subroutine fmin_jam
!----------------------------------------
Subroutine fmin2_jam(x_in,fx,i_in)
!Version that internally fits M/L ratio optimally. Don't put M/L in free list. 
!Put it in hold list, but it won't be used.
!Might have better convergence properties. Slower, as two calls are 
!required to JAM.
use mod_jammin
Implicit None
!call a JAM model for a single set of parameters. Considers that many fibers
!make up a bin. Needs a map of fiber/bin ownership to weights to do this, but 
!this code uses those weights. Generates a single model. Similar version for 
!chisq feeding to lancelot.
Integer::i,j,k,ndum,nmax,i_in,npot,ngauss
Parameter(nmax=15)
real::x_in(:),rjunk1,rjunk2,rjunk3,xcomb(6)
real::fx
Real::ML_now,inc_now,beta_now,cval,M200,dm_gamma,mbh_now,surf_tmp(nmax),&
sigma_tmp(nmax),qobs_tmp(nmax),rmin,rmax
Real,allocatable,dimension(:)::surf_pot1,sigma_pot1,qobs_pot1,surf_pot2,sigma_pot2,&
qobs_pot2,tmp_sum1,tmp_sum2,vrmsmodel1,vrmsmodel2,vfib,beta_lum
Parameter(mbh_now=0.0)

!ML_now=x_in(1)
!inc_now=x_in(2)
!beta_now=x_in(3)
!cval=x_in(4)
!M200=10.**x_in(5)
!dm_gamma=x_in(6)
!Method to map arbitrary minization
Do i=1,nfree
   xcomb(ifree(i))=x_in(i)
End Do
if (nhold.gt.0) then
   Do i=1,nhold
      xcomb(ihold(i))=xhold(i)
   End Do
end if
ML_now=xcomb(1)
inc_now=xcomb(2)
beta_now=xcomb(3)
cval=xcomb(4)
M200=10.**xcomb(5)
dm_gamma=xcomb(6)

fx=0.0
!write(*,*) "Iteration ", x_in(:)
pc_now=distance_now*my_pi/0.648


rmin=max(pc_now*pixsize/10.0,10.0) !min 10 pc size
rmax=pc_now*sqrt(maxval(xfib(:)**2+yfib(:)**2))*10.0
call gNFW_mge(cval,M200,dm_gamma,rmin,rmax,ngauss,surf_tmp,sigma_tmp,qobs_tmp)
npot=nlum+ngauss
Allocate(surf_pot1(ngauss),sigma_pot1(ngauss),qobs_pot1(ngauss),&
surf_pot2(nlum),sigma_pot2(nlum),qobs_pot2(nlum),tmp_sum1(ndata),tmp_sum2(ndata),&
vrmsmodel1(ndata),vrmsmodel2(ndata),vfib(nfib),beta_lum(nlum))
surf_pot2(1:nlum)=surf_lum(1:nlum)
sigma_pot2(1:nlum)=sigma_lum(1:nlum)
qobs_pot2(1:nlum)=qobs_lum(1:nlum)
beta_lum(:)=beta_now
call jam_axisymmetric_rms(nlum,surf_lum,sigma_lum,qobs_lum,&
nlum,surf_pot2,sigma_pot2,qobs_pot2,inc_now,mbh_now,distance_now,nfib,&
xfib,yfib,vfib,beta_lum,sigmapsf,pixsize)
tmp_sum1(:)=0.0
tmp_sum2(:)=0.0
Do i=1,nfib
   tmp_sum1(indfib(i))=tmp_sum1(indfib(i))+vfib(i)**2*gamfib(i)
   tmp_sum2(indfib(i))=tmp_sum2(indfib(i))+gamfib(i)
End Do
Do i=1,ndata
   vrmsmodel2(i)=sqrt(tmp_sum1(i)/tmp_sum2(i))
End Do
surf_pot1(1:ngauss)=surf_tmp(1:ngauss)
sigma_pot1(1:ngauss)=sigma_tmp(1:ngauss)/pc_now
qobs_pot1(1:ngauss)=qobs_tmp(1:ngauss)
call jam_axisymmetric_rms(nlum,surf_lum,sigma_lum,qobs_lum,&
ngauss,surf_pot1,sigma_pot1,qobs_pot1,inc_now,mbh_now,distance_now,nfib,&
xfib,yfib,vfib,beta_lum,sigmapsf,pixsize)
tmp_sum1(:)=0.0
tmp_sum2(:)=0.0
Do i=1,nfib
   tmp_sum1(indfib(i))=tmp_sum1(indfib(i))+vfib(i)**2*gamfib(i)
   tmp_sum2(indfib(i))=tmp_sum2(indfib(i))+gamfib(i)
End Do
Do i=1,ndata
   vrmsmodel1(i)=sqrt(tmp_sum1(i)/tmp_sum2(i))
End Do
rjunk1=0.0
rjunk2=0.0
Do i=1,ndata
   rjunk1=rjunk1+(vdata(i)**2-vrmsmodel1(i)**2)/vrmsmodel2(i)**2/evdata(i)**4
   rjunk2=rjunk2+1.0/evdata(i)**4
End Do
ML_best=max(int_MLmin,min(int_MLmax,rjunk1/rjunk2))
Do i=1,ndata
   rjunk3=sqrt(ML_best*vrmsmodel2(i)**2+vrmsmodel1(i)**2)
   fx=fx+((rjunk3-vdata(i))/evdata(i))**2
End Do
fx=fx/real(ndata)
!write(*,*) x_in(:), fx
Deallocate(surf_pot1,sigma_pot1,qobs_pot1,surf_pot2,sigma_pot2,qobs_pot2&
,tmp_sum1,tmp_sum2,vrmsmodel1,vrmsmodel2,vfib,beta_lum)
return
end subroutine fmin2_jam
!----------------------------------------
Subroutine fmin3_jam(x_in,fx,i_in)
!Version that also scaled c200 by M200 per Duffy+08. Keep cval in the hold 
!matrix, but won't be used on input.
!Version that internally fits M/L ratio optimally. Don't put M/L in free list. 
!Put it in hold list, but it won't be used.
!Might have better convergence properties. Slower, as two calls are 
!required to JAM.
use mod_jammin
Implicit None
!call a JAM model for a single set of parameters. Considers that many fibers
!make up a bin. Needs a map of fiber/bin ownership to weights to do this, but 
!this code uses those weights. Generates a single model. Similar version for 
!chisq feeding to lancelot.
Integer::i,j,k,ndum,nmax,i_in,npot,ngauss
Parameter(nmax=15)
real::x_in(:),rjunk1,rjunk2,rjunk3,xcomb(6)
real::fx
Real::ML_now,inc_now,beta_now,cval,M200,dm_gamma,mbh_now,surf_tmp(nmax),&
sigma_tmp(nmax),qobs_tmp(nmax),rmin,rmax
Real,allocatable,dimension(:)::surf_pot1,sigma_pot1,qobs_pot1,surf_pot2,sigma_pot2,&
qobs_pot2,tmp_sum1,tmp_sum2,vrmsmodel1,vrmsmodel2,vfib,beta_lum
Parameter(mbh_now=0.0)

!ML_now=x_in(1)
!inc_now=x_in(2)
!beta_now=x_in(3)
!cval=x_in(4)
!M200=10.**x_in(5)
!dm_gamma=x_in(6)
!Method to map arbitrary minization
Do i=1,nfree
   xcomb(ifree(i))=x_in(i)
End Do
if (nhold.gt.0) then
   Do i=1,nhold
      xcomb(ihold(i))=xhold(i)
   End Do
end if
ML_now=xcomb(1)
inc_now=xcomb(2)
beta_now=xcomb(3)
M200=10.**xcomb(5)
cval=9.0*(M200/18.57d12)**(-0.13) !Bullock+01
!cval=9.6*(M200/1.857d12)**(-0.075) !Klypin+11
!cval=5.47*(M200/2.857d12)**(-0.097) !Duffy+08
c_best=cval
dm_gamma=xcomb(6)

fx=0.0
!write(*,*) "Iteration ", x_in(:)
pc_now=distance_now*my_pi/0.648


rmin=max(pc_now*pixsize/10.0,10.0) !min 10 pc size
rmax=pc_now*sqrt(maxval(xfib(:)**2+yfib(:)**2))*10.0
call gNFW_mge(cval,M200,dm_gamma,rmin,rmax,ngauss,surf_tmp,sigma_tmp,qobs_tmp)
npot=nlum+ngauss
Allocate(surf_pot1(ngauss),sigma_pot1(ngauss),qobs_pot1(ngauss),&
surf_pot2(nlum),sigma_pot2(nlum),qobs_pot2(nlum),tmp_sum1(ndata),tmp_sum2(ndata),&
vrmsmodel1(ndata),vrmsmodel2(ndata),vfib(nfib),beta_lum(nlum))
surf_pot2(1:nlum)=surf_lum(1:nlum)
sigma_pot2(1:nlum)=sigma_lum(1:nlum)
qobs_pot2(1:nlum)=qobs_lum(1:nlum)
beta_lum(:)=beta_now
call jam_axisymmetric_rms(nlum,surf_lum,sigma_lum,qobs_lum,&
nlum,surf_pot2,sigma_pot2,qobs_pot2,inc_now,mbh_now,distance_now,nfib,&
xfib,yfib,vfib,beta_lum,sigmapsf,pixsize)
tmp_sum1(:)=0.0
tmp_sum2(:)=0.0
Do i=1,nfib
   tmp_sum1(indfib(i))=tmp_sum1(indfib(i))+vfib(i)**2*gamfib(i)
   tmp_sum2(indfib(i))=tmp_sum2(indfib(i))+gamfib(i)
End Do
Do i=1,ndata
   vrmsmodel2(i)=sqrt(tmp_sum1(i)/tmp_sum2(i))
End Do
surf_pot1(1:ngauss)=surf_tmp(1:ngauss)
sigma_pot1(1:ngauss)=sigma_tmp(1:ngauss)/pc_now
qobs_pot1(1:ngauss)=qobs_tmp(1:ngauss)
call jam_axisymmetric_rms(nlum,surf_lum,sigma_lum,qobs_lum,&
ngauss,surf_pot1,sigma_pot1,qobs_pot1,inc_now,mbh_now,distance_now,nfib,&
xfib,yfib,vfib,beta_lum,sigmapsf,pixsize)
tmp_sum1(:)=0.0
tmp_sum2(:)=0.0
Do i=1,nfib
   tmp_sum1(indfib(i))=tmp_sum1(indfib(i))+vfib(i)**2*gamfib(i)
   tmp_sum2(indfib(i))=tmp_sum2(indfib(i))+gamfib(i)
End Do
Do i=1,ndata
   vrmsmodel1(i)=sqrt(tmp_sum1(i)/tmp_sum2(i))
End Do
rjunk1=0.0
rjunk2=0.0
Do i=1,ndata
   rjunk1=rjunk1+(vdata(i)**2-vrmsmodel1(i)**2)/vrmsmodel2(i)**2/evdata(i)**4
   rjunk2=rjunk2+1.0/evdata(i)**4
End Do
ML_best=max(int_MLmin,min(int_MLmax,rjunk1/rjunk2))
Do i=1,ndata
   rjunk3=sqrt(ML_best*vrmsmodel2(i)**2+vrmsmodel1(i)**2)
   fx=fx+((rjunk3-vdata(i))/evdata(i))**2
End Do
fx=fx/real(ndata)
!write(*,*) x_in(:), c_best, ML_best, fx
Deallocate(surf_pot1,sigma_pot1,qobs_pot1,surf_pot2,sigma_pot2,qobs_pot2&
,tmp_sum1,tmp_sum2,vrmsmodel1,vrmsmodel2,vfib,beta_lum)
return
end subroutine fmin3_jam
!----------------------------------------
Subroutine fmin4_jam(x_in,fx,i_in)
use mod_jammin
Implicit None
!Fixes c/M relation, but doesn't fit M/L
!call a JAM model for a single set of parameters. Considers that many fibers
!make up a bin. Needs a map of fiber/bin ownership to weights to do this, but 
!this code uses those weights. Generates a single model. Similar version for 
!chisq feeding to lancelot.
Integer::i,j,k,ndum,nmax,i_in,npot,ngauss
Parameter(nmax=15)
real::x_in(:), rjunk1,xcomb(6)
real::fx
Real::ML_now,inc_now,beta_now,cval,M200,dm_gamma,mbh_now,surf_tmp(nmax),&
sigma_tmp(nmax),qobs_tmp(nmax),rmin,rmax
Real,allocatable,dimension(:)::surf_pot,sigma_pot,qobs_pot,tmp_sum1,tmp_sum2,&
vrmsmodel,vfib,beta_lum
Parameter(mbh_now=0.0)

!ML_now=x_in(1)
!inc_now=x_in(2)
!beta_now=x_in(3)
!cval=x_in(4)
!M200=10.**x_in(5)
!dm_gamma=x_in(6)
!Method to map arbitrary minization
Do i=1,nfree
   xcomb(ifree(i))=x_in(i)
End Do
if (nhold.gt.0) then
   Do i=1,nhold
      xcomb(ihold(i))=xhold(i)
   End Do
end if
ML_now=xcomb(1)
inc_now=xcomb(2)
beta_now=xcomb(3)
M200=10.**xcomb(5)
cval=9.0*(M200/18.57d12)**(-0.13) !Bullock+01
!cval=9.6*(M200/1.857d12)**(-0.075) !Klypin+11
!cval=5.47*(M200/2.857d12)**(-0.097) !Duffy+08
c_best=cval
dm_gamma=xcomb(6)

fx=0.0
!write(*,*) "Iteration ", x_in(:)
pc_now=distance_now*my_pi/0.648


rmin=max(pc_now*pixsize/10.0,10.0) !min 10 pc size
rmax=pc_now*sqrt(maxval(xfib(:)**2+yfib(:)**2))*10.0
call gNFW_mge(cval,M200,dm_gamma,rmin,rmax,ngauss,surf_tmp,sigma_tmp,qobs_tmp)
npot=nlum+ngauss
Allocate(surf_pot(npot),sigma_pot(npot),qobs_pot(npot),&
tmp_sum1(ndata),tmp_sum2(ndata),vrmsmodel(ndata),vfib(nfib),beta_lum(nlum))
surf_pot(1:nlum)=ML_now*surf_lum(1:nlum)
sigma_pot(1:nlum)=sigma_lum(1:nlum)
qobs_pot(1:nlum)=qobs_lum(1:nlum)
beta_lum(:)=beta_now
i=nlum+1
j=nlum+ngauss
surf_pot(i:j)=surf_tmp(1:ngauss)
sigma_pot(i:j)=sigma_tmp(1:ngauss)/pc_now
qobs_pot(i:j)=qobs_tmp(1:ngauss)
call jam_axisymmetric_rms(nlum,surf_lum,sigma_lum,qobs_lum,&
npot,surf_pot,sigma_pot,qobs_pot,inc_now,mbh_now,distance_now,nfib,&
xfib,yfib,vfib,beta_lum,sigmapsf,pixsize)
tmp_sum1(:)=0.0
tmp_sum2(:)=0.0
Do i=1,nfib
   tmp_sum1(indfib(i))=tmp_sum1(indfib(i))+vfib(i)**2*gamfib(i)
   tmp_sum2(indfib(i))=tmp_sum2(indfib(i))+gamfib(i)
End Do
Do i=1,ndata
   vrmsmodel(i)=sqrt(tmp_sum1(i)/tmp_sum2(i))
   fx=fx+((vrmsmodel(i)-vdata(i))/evdata(i))**2
End Do
fx=fx/real(ndata)
!write(*,*) x_in(:), fx
Deallocate(surf_pot,sigma_pot,qobs_pot,tmp_sum1,tmp_sum2,vrmsmodel,vfib,beta_lum)
return
end subroutine fmin4_jam
!----------------------------------------
