module mod_mge_fit_1d
  implicit none
  integer::ngauss,ndata
  Real,allocatable,dimension(:)::x1,y1,x_con,df_con
  Real::my_pi,aobj
  Parameter(my_pi=3.14159265259)
End module mod_mge_fit_1d
!----------------------------------------
Subroutine mge_fit_1d(ndata_in,x1_in,y1_in,ngauss_in,sol,ngauss_out)
Use mod_mge_fit_1d
USE Constrained_minimization
use common_cnmn1
!use LANCELOT_simple_double
Implicit None
Integer::i,j,k,i_status,keep_ngauss,ngauss_in,ndata_in,ngauss_out,&
n1,n2,n3,n4,n5,nlim
Integer,dimension(ngauss_in)::sig_index
Real,dimension(ndata_in)::x1_in,y1_in
Real,dimension(2)::rbounds
Real,dimension(2,ngauss_in)::sol
Real,dimension(2*ngauss_in)::sol_fit,sol_min,sol_max
Real::outer_slope,tmp_chisq,ylogmin,ylogmax,rmin,rmax,rjunk1
Real,allocatable,dimension(:)::g_con,scal_con,s_con,g1_con,&
g2_con,c_con
Real,allocatable,dimension(:,:)::a_con,b_con
Integer,allocatable,dimension(:)::isc_con,ic_con,ms1_con
Parameter(outer_slope=2.0)
External:: Fminvel,Gminvel

!IPRINT = 2
IPRINT = 0
NDV = 2*ngauss_in
ITMAX = 200
NCON = 0
NSIDE=1
ICNDIR=0
NSCAL=NDV+1
NFDG=0
LINOBJ=0
ITRM = 3
FDCH=1.d-3
FDCHM=1.d-4
CT=-0.1
CTMIN=0.0004
CTL=-0.01
CTLMIN=0.001
THETA=1.0
DELFUN=1.0d-4
DABFUN=1.0d-3
n1=ndv
n2=ncon+ndv
n3=ndv+1
n4=max(n3,ndv)
n5=2*n4
nlim=itmax*(ndv+5)
Allocate(x_con(n1),g_con(n2),scal_con(n1),df_con(n1),a_con(n1,n3),s_con(n1),g1_con(n2),&
g2_con(n2),b_con(n3,n3),c_con(n4),isc_con(n2),ic_con(n3),ms1_con(n5))
ngauss=ngauss_in
ndata=ndata_in
allocate(x1(ndata))
allocate(y1(ndata))
!x1(i)=10.**(rmin+(rmax-rmin)*real(i-1)/real(ndata-1))
x1(:)=x1_in(:)
y1(:)=y1_in(:)
ylogmin=log10(y1(ndata))
ylogmax=log10(y1(1))
rbounds(1)=log10(x1(1))
rbounds(2)=log10(x1(ndata)/sqrt(outer_slope))

Do i=1,ngauss
   sol_fit(i)=10.0**(2.0*(rbounds(1)+(rbounds(2)-rbounds(1))*real(i-1)/real(ngauss)))
   sol_min(i)=10.0**(2.0*rbounds(1))
   sol_max(i)=10.0**(2.0*rbounds(2))
   j=min(ndata-1,max(1,floor(ndata*real(i-1)/ngauss)))
   sol_fit(i+ngauss)=0.5*(y1(j)+y1(j+1))*sqrt(2.0*my_pi*sol_fit(i))
!   sol_fit(i+ngauss)=10.**(ylogmax+(ylogmin-ylogmax)*real(i-1)/real(ngauss))*&
!sqrt(sol_fit(i))
   sol_min(i+ngauss)=0.0
   sol_max(i+ngauss)=max(1.d20,10.*sol_fit(i+ngauss))
End Do
x_con(:)=sol_fit(:)
igoto=0
!call lancelot_simple(2*ngauss,sol_fit,tmp_chisq,i_status,&
!MY_FUN=Fminvel,MY_GRAD=Gminvel,BL=sol_min,BU=sol_max,print_level=-1)
x_con(:)=sol_fit(:)
Do i=1,nlim
   call conmin(x_con,sol_min,sol_max,G_con,SCAL_con,DF_con,A_con,S_con,G1_con,G2_con,B_con,&
C_con,ISC_con,IC_con,MS1_con,N1,N2,N3,N4,N5)
   call conmin_Fminvel()
   obj=aobj
   if (igoto.eq.0) exit
End Do
sol_fit(:)=x_con(:)
!call lancelot_simple(2*ngauss,sol_fit,tmp_chisq,i_status,&
!MY_FUN=Fminvel,MY_GRAD=Gminvel,BL=sol_min,BU=sol_max,print_level=-1)

call indexx(ngauss,sol_fit,sig_index)
keep_ngauss=0
sol(1,:)=0.0
sol(2,:)=0.0
Do i=1,ngauss
   if (sol_fit(sig_index(i)+ngauss).gt.0.0) then
      keep_ngauss=keep_ngauss+1
      sol(1,keep_ngauss)=sqrt(sol_fit(sig_index(i)))
      sol(2,keep_ngauss)=sol_fit(sig_index(i)+ngauss)
   end if
End Do
deallocate(x1)
deallocate(y1)
Deallocate(x_con,g_con,scal_con,df_con,a_con,s_con,g1_con,&
g2_con,b_con,c_con,isc_con,ic_con,ms1_con)
ngauss_out=keep_ngauss
Return
End Subroutine mge_fit_1d
!----------------------------------------
subroutine mk_mge_model(ngauss,sol,ndata,x_in,y_model)
integer::i_in,i,j,k,ngauss,ndata
real,dimension(2,ngauss)::sol
real,dimension(ndata)::x_in,y_model
real::rjunk1,my_pi
Parameter(my_pi=3.14159265259)
Do i=1,ndata
   rjunk1=0.0
   Do j=1,ngauss
      rjunk1=rjunk1+sol(2,j)*exp(-x_in(i)**2/(2.0*sol(1,j)**2))/&
sqrt(2.0*my_pi)/sol(1,j)
   End Do
   y_model(i)=rjunk1
End Do
end subroutine mk_mge_model
!----------------------------------------
subroutine conmin_Fminvel()
Use mod_mge_fit_1d
USE Constrained_minimization
use common_cnmn1
integer::i_in,i,j,k
real::rjunk1
real::fx
!Fits 1d Gaussian as step to transform fit to 3d 
!function to 2d output. sigma terms are really sigma^2.
if (info.lt.2) then
!objective function
   
!   Open(91,file="erer.txt",access='append')
   fx=0.0
   Do i=1,ndata
      rjunk1=0.0
      Do j=1,ngauss
         rjunk1=rjunk1+x_con(j+ngauss)*exp(-x1(i)**2/(2.0*x_con(j)))/&
   sqrt(2.0*my_pi*x_con(j))
      End Do
      fx=fx+((rjunk1-y1(i))/y1(i))**2
!      write(91,*) x1(i), y1(i), rjunk1
   End Do
!   write(91,*) 
!   write(91,*) 
!   Close(91)
   aobj=fx
else
!gradient
   df_con(:)=0.0
   Do i=1,ndata
      rjunk1=0.0
      Do j=1,ngauss
         rjunk1=rjunk1+x_con(j+ngauss)*exp(-x1(i)**2/(2.0*x_con(j)))/&
sqrt(2.0*my_pi*x_con(j))
      End Do
      Do j=1,ngauss
         df_con(j)=df_con(j)+2.0*(rjunk1-y1(i))*x_con(j+ngauss)*(exp(-x1(i)**2/&
(2.0*x_con(j)))&
*x1(i)**2/(2.0*x_con(j)**2)/sqrt(2.0*my_pi*x_con(j))-0.5*exp(-x1(i)**2/(2.0*x_con(j)))&
*(2.0*my_pi)/(2.0*my_pi*x_con(j))**(1.5))/y1(i)**2
         df_con(j+ngauss)=df_con(j+ngauss)+2.0*(rjunk1-y1(i))*exp(-x1(i)**2/(2.0*x_con(j)))/&
sqrt(2.0*my_pi*x_con(j))/y1(i)**2
      End Do
   End Do
end if

end subroutine conmin_Fminvel
!----------------------------------------
subroutine Fminvel(x_in,fx,i_in)
Use mod_mge_fit_1d
integer::i_in,i,j,k
real::x_in(:), rjunk1
real::fx
!Fits 1d Gaussian as step to transform fit to 3d 
!function to 2d output. sigma terms are really sigma^2.
fx=0.0
Do i=1,ndata
   rjunk1=0.0
   Do j=1,ngauss
      rjunk1=rjunk1+x_in(j+ngauss)*exp(-x1(i)**2/(2.0*x_in(j)))/&
sqrt(2.0*my_pi*x_in(j))
   End Do
   fx=fx+((rjunk1-y1(i))/y1(i))**2
End Do
end subroutine Fminvel
!----------------------------------------
subroutine Gminvel(x_in,dfx,i_in)
Use mod_mge_fit_1d
integer::i_in,i,j,k
real::x_in(:), rjunk1, rjunk2
real::dfx(:)

dfx(:)=0.0
Do i=1,ndata
   rjunk1=0.0
   Do j=1,ngauss
      rjunk1=rjunk1+x_in(j+ngauss)*exp(-x1(i)**2/(2.0*x_in(j)))/&
sqrt(2.0*my_pi*x_in(j))
   End Do
   Do j=1,ngauss
      dfx(j)=dfx(j)+2.0*(rjunk1-y1(i))*x_in(j+ngauss)*(exp(-x1(i)**2/(2.0*x_in(j)))&
*x1(i)**2/(2.0*x_in(j)**2)/sqrt(2.0*my_pi*x_in(j))-0.5*exp(-x1(i)**2/(2.0*x_in(j)))&
*(2.0*my_pi)/(2.0*my_pi*x_in(j))**(1.5))/y1(i)**2
      dfx(j+ngauss)=dfx(j+ngauss)+2.0*(rjunk1-y1(i))*exp(-x1(i)**2/(2.0*x_in(j)))/&
sqrt(2.0*my_pi*x_in(j))/y1(i)**2
   End Do
End Do
end subroutine Gminvel
