import numpy as np
import math

cdef extern:
    void c_sub_eval_gas(double *gas_pars, int *n_data, double *x1_pos, 
       double *y1_pos, double *vobs, double *evobs, int *n_mge, double *mge_surf, double *mge_sigma, 
       double *mge_qobs, int *n_mge_HI, double *mge_surf_HI, double *mge_sigma_HI,
       double *mge_qobs_HI, double *distance, double *incl0, double *min_inc, double *PA0, double *vsys0, 
       double *out)

def sub_eval_gas(double[::1] gas_pars not None,
                   double[::1] x1_pos not None, 
                   double[::1] y1_pos not None, double[::1] vobs not None, double[::1] evobs not None,
                   double[::1] mge_surf not None, double[::1] mge_sigma not None,
                   double[::1] mge_qobs not None, double[::1] mge_surf_HI not None, 
                   double[::1] mge_sigma_HI not None, double[::1] mge_qobs_HI not None, 
                   double distance, double incl0, double PA0, 
                   double vsys0, double out):
    # initialize the output variable with cython so Python manages the
    # memory:
    cdef int n_data, n_mge, n_mge_HI
    cdef double min_inc
    out = 0.0
    outp1 = 0.0
    n_data=len(x1_pos)
    n_mge=len(mge_surf)
    n_mge_HI=len(mge_surf_HI)
#    min_inc=math.degrees(math.acos(np.sqrt((np.min(mge_qobs)**2-0.0025)/0.9975)))
    min_inc=math.degrees(math.acos(np.min(np.concatenate([mge_qobs,mge_qobs_HI]))))+1.0E-2
    if gas_pars[3] > 0.35 and gas_pars[3] < 10.0 and gas_pars[2] < 2.0 and gas_pars[1] > 2.0 and gas_pars[1] < 40.0 and gas_pars[2] >= 0.0 and gas_pars[9] > 0.0:
#    if gas_pars[4] >= min_inc and gas_pars[2] < 2.0 and gas_pars[2] >= 0.0:
       c_sub_eval_gas(&gas_pars[0],&n_data,&x1_pos[0],&y1_pos[0],&vobs[0],&evobs[0],&n_mge,
       &mge_surf[0],&mge_sigma[0],&mge_qobs[0],&n_mge_HI,&mge_surf_HI[0],&mge_sigma_HI[0],&mge_qobs_HI[0],
       &distance,&incl0,&min_inc,&PA0,&vsys0,&outp1)
    else:
#       outp1=-1.E20
       return -np.inf
    out = outp1
    return out
