Module rot_cur_mod
Implicit None
Real::cneg2_now,gamma_now
Real::mge_e2,mge_s2,mge_r2,mge_z2,my_pi
Parameter(my_pi=3.1415926535897932)
End Module
!-----------------------------------------------
Subroutine func_sing_velrot(ma,a,n_pos,r_pos,v_dm,distance)
!Calculates the DM 
!velocity, and outputs the combined (in the disk plane). a should be 
!M200, c, gamma, and M/L
use rot_cur_mod
IMPLICIT NONE
INTEGER::ma,n_pos,i
REAL::a(ma),r_pos(n_pos),v_dm(n_pos),rjunk1,rjunk2,rneg2,rot_DM,rvir,V200,x_in,M200,cval,kpc1,distance
External::func_velrot

kpc1=distance*my_pi/0.648d3
!parameters are M200, cneg2, gamma, M/L, inc
!Mvir===delvir*rho_crit*(4/3)*pi*Rvir^3
!delvir===200; rho_crit===3*H^2/(8*pi*G)=9.47d-30 g/cm^3= 139.866 Msol/kpc^3
!Msol= 1.9891d33 g
!kpc= 3.0856d21 cm
M200=a(1)
rvir=(M200/117174.0)**(1.0/3.0) !in kpc and Msol
!cval=9.0*(M200/18.57d12)**(-0.13) !Bullock+01
gamma_now=a(3)
cval=a(2)/(2.0-gamma_now) !really c_-2
rneg2=rvir/cval

V200=0.7*rvir
!cneg2===Rvir/rneg2
!x===R/rneg2
call qromb(func_velrot,0.,cval,rjunk2)
Do i=1,n_pos
   x_in=kpc1*r_pos(i)/rneg2
!(Vvir/Rvir)^2===h^2*(delvir/200), for km/s and kpc; h===0.7
   call qromb(func_velrot,0.,x_in,rjunk1)
   rot_DM=V200*sqrt((cval/x_in)*rjunk1/rjunk2)
   v_dm(i)=rot_DM
!   v_circ(i)=sqrt(v_HI(i)**2+a(4)*v_stel(i)**2+rot_DM**2)
End Do
Return
End Subroutine
!-----------------------------------------------
Function func_velrot(x_fitting)
use rot_cur_mod
!Evaluates the rotation curve function from Dutton '05, mu
IMPLICIT NONE
REAL::x_fitting,func_velrot
func_velrot=x_fitting**(2.-gamma_now)*(1.+(2.-gamma_now)*x_fitting)**(gamma_now-3.)
Return
End Function
!-----------------------------------------------
subroutine FUN(x_in,fx,i_in)
integer::i_in,i,j,k
real::x_in(:), rjunk1
real::fx
fx=1.0
Return
end subroutine FUN
!-----------------------------------------------
Subroutine deproj_pos(n_pos,x1,y1,x0,y0,PA,incl,r1,psi1)
!Deprojects positions following vdV's IDL code
!Projected velocities will be v(r)*cos(psi)*sin(i)
use rot_cur_mod
Implicit None
Integer::i,n_pos
Real::x1(n_pos),y1(n_pos),x0,y0,PA,incl,r1(n_pos),psi1(n_pos),sinpa,cospa,xp(n_pos),yp(n_pos),cosi
sinpa=sin(PA*(my_pi/180.0))
cospa=cos(PA*(my_pi/180.0))
cosi=cos(incl*(my_pi/180.0))
Do i=1,n_pos
   xp(i)=-sinpa*(x1(i)-x0)+cospa*(y1(i)-y0)
   yp(i)=-(cospa*(x1(i)-x0)+sinpa*(y1(i)-y0))
   r1(i)=sqrt(xp(i)**2+(yp(i)/cosi)**2)
   psi1(i)=0.0
   if (r1(i).gt.1.d-10) psi1(i)=acos(xp(i)/r1(i))
   if (yp(i).lt.0.0) psi1(i)=2.0*my_pi-psi1(i)
End do
Return
End subroutine
!-----------------------------------------------
Subroutine mge_circ_vel(n_mge,surf_pc,sigma_arcsec,qobs,inc_deg,distance,n_rad,rad,vcirc)
use rot_cur_mod
Implicit None
Integer::n_mge,n_rad,i,j,k
Real::surf_pc(n_mge),sigma_arcsec(n_mge),qobs(n_mge),inc_deg,distance,rad(n_rad),vcirc(n_rad),&
pc,rcirc(n_rad),sigma(n_rad),inc1,qintr(n_mge),dens(n_mge),zcirc(n_rad)

pc=distance*my_pi/0.648
Do i=1,n_rad
   rcirc(i)=rad(i)*pc
   zcirc(i)=0.0
End Do
inc1=inc_deg*my_pi/180.0
Do i=1,n_mge
   sigma(i)=sigma_arcsec(i)*pc
   qintr(i)=qobs(i)**2-(cos(inc1))**2
   if (qintr(i).le.0.0) then
      write(*,*) "Error in deprojection", inc_deg,  qobs(i), cos(inc1)
      stop
!      qintr(i)=0.0
   else
      qintr(i)=sqrt(qintr(i))/sin(inc1)
   endif
   dens(i)=surf_pc(i)*qobs(i)/(qintr(i)*sigma(i)*sqrt(2.0*my_pi))
End Do
call mgevc_accR(n_rad,rcirc,zcirc,vcirc,n_mge,dens,sigma,qintr)
Do i=1,n_rad
   vcirc(i)=sqrt(rcirc(i)*abs(vcirc(i)))
End Do
Return
End Subroutine
!-----------------------------------------------
Subroutine mgevc_accR(n_rad,r_in,z_in,acc1,n_mge,dens,sigma,qintr)
use rot_cur_mod
Implicit None
Integer::n_mge,n_rad,i,j,k,limit,lenw,leniw,ier,last
Real::r_in(n_rad),z_in(n_rad),dens(n_mge),sigma(n_mge),qintr(n_mge),&
potall(n_mge),mgepot(n_rad),acc1(n_rad),e2(n_mge),s2(n_mge),r2(n_rad),&
z2(n_rad),d2(n_rad),tmp_e,eint1,epsabs,epsrel,abserr
Parameter(limit=20,lenw=920,leniw=60)
Integer::iwork(leniw)
Real::work(lenw)
external::evc_dRRH
epsabs = 0.0d-3
epsrel = 1.0d-4
ier=0

Do k=1,n_mge
   e2(k) = 1.0 - qintr(k)**2
   s2(k) = sigma(k)**2
End Do
Do k=1,n_rad
   r2(k) = r_in(k)**2
   z2(k) = z_in(k)**2
   d2(k) = r2(k) + z2(k)
End Do

Do k=1,n_rad
   Do j=1,n_mge
!Allows prolate halos by commenting out.
!      if (d2(k).lt.s2(j)/57600.0) then
!         tmp_e = sqrt(e2(j))
!         potall(j) = (asin(tmp_e)/tmp_e - qintr(j))/(2.0*e2(j)*s2(j)) 
!      else if (d2(k).lt.s2(j)*60025.0) then
      if (d2(k).lt.s2(j)*60025.0) then
         mge_e2=e2(j)
         mge_s2=s2(j)
         mge_z2=z2(k)
         mge_r2=r2(k)
         eint1=0.0
         call dqxgs(evc_dRRH,0.0,1.0,epsabs,epsrel,eint1,&
abserr,ier,limit,leniw,lenw,last,iwork,work)
         if (ier /= 0 ) then
            write(*,*) "dqxgs error in subroutine janis2:",ier
            stop
         endif
!         call qromb(evc_dRRH,0.0,1.0,eint1)
         potall(j) = eint1
      else
         potall(j) = sqrt(my_pi/2.0)*sigma(j)/d2(k)**1.5
      endif
   End Do
   mgepot(k) = sum(s2(:)*qintr(:)*dens(:)*potall(:))
   acc1(k)=-4.0*r_in(k)*my_pi*0.00430237*mgepot(k)
End Do
Return
end Subroutine
!-----------------------------------------------
Function evc_dRRH(u_in)
use rot_cur_mod
!Evaluates the MGE integral per Cappellari, Eq A3
IMPLICIT NONE
REAL::u_in,evc_dRRH,u2,p2,us2
u2 = u_in**2
p2 = 1.0 - mge_e2*u2
us2 = u2/mge_s2
evc_dRRH=exp(-0.5*us2*(mge_r2+mge_z2/p2))*us2/sqrt(p2)
Return
End Function
!-----------------------------------------------
!Solves for DM MGE terms intrinsic axis ratios to fit a smooth function. 
!The qterms are 1: qlow, 2: qhigh, and 3: rscale("). 
Subroutine solve_qmge(n_mge,l_mge,sig_mge,q_mge,qlow,qhigh,qscale)
Implicit None
Integer::n_mge,n_rad,i,j,k
Real::isqr2pic
Parameter (n_rad=101,isqr2pic=0.063493636)
Integer::i_svd(n_mge),ijunk1
Real::l_mge(n_mge),sig_mge(n_mge),q_mge(n_mge),qlow,qhigh,qscale,&
r1(n_rad),q1(n_rad),rmin,rmax,rdx,a_svd(n_rad,n_mge),w_svd(n_mge),&
v_svd(n_mge,n_mge),b_svd(n_rad),x_svd(n_mge),rnorm


rmin=log10(minval(sig_mge)/2.0)
rmax=log10(maxval(sig_mge)*2.0)
rdx=(rmax-rmin)/real(n_rad-1)

Do i=1,n_rad
   r1(i)=10.**(rmin+real(i-1)*rdx)
   q1(i)=qlow*exp(-r1(i)/qscale)+qhigh*(1.0-exp(-r1(i)/qscale))
End Do

Do i=1,n_rad
   b_svd(i)=0.0
   Do k=1,n_mge
      b_svd(i)=b_svd(i)+l_mge(k)*exp(-0.5*(r1(i)/sig_mge(k))**2)*isqr2pic/(sig_mge(k))**3
   End Do
   Do j=1,n_mge
      a_svd(i,j)=q1(i)*l_mge(j)*exp(-0.5*(r1(i)/sig_mge(j))**2)*isqr2pic/(sig_mge(j))**3
   End Do
End Do
call nnls(a_svd,n_rad,n_mge,b_svd,x_svd,rnorm,w_svd,i_svd,ijunk1)
!call svdcmp(a_svd,n_rad,n_mge,n_rad,n_mge,w_svd,v_svd)
!call svbksb(a_svd,w_svd,v_svd,n_rad,n_mge,n_rad,n_mge,b_svd,x_svd)
Do i=1,n_mge
   if (x_svd(i).gt.20.0) then
      q_mge(i)=0.05
   else if (x_svd(i).lt.1.d-2) then
      q_mge(i)=1.0
   else
      q_mge(i)=1./x_svd(i)
   endif
End Do
Return
end Subroutine
